@@ -15,24 +15,18 @@
 
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
 // 27.4.3 fpos
 
+// { dg-do run { xfail *-*-* } }
+
+#include <typeinfo>
+#include <limits>
 #include <iterator>
 #include <testsuite_hooks.h>
 
-class Fred
-{
-public:
-  Fred(bool)
-  { }
-};
-
-void barney(Fred)
-{ }
-
 // libstdc++/14320
 void test01()
 {
@@ -40,9 +34,24 @@ void test01()
   bool test __attribute__((unused)) = true;
 
   typedef istreambuf_iterator<char>::difference_type Distance;
+
+  bool found = false;
+  if (typeid(Distance) == typeid(long int))
+    found = true;
+  if (typeid(Distance) == typeid(int))
+    found = true;
+  if (typeid(Distance) == typeid(short int))
+    found = true;
+  if (typeid(Distance) == typeid(signed char))
+    found = true;
+  if (numeric_limits<char>::is_signed &&
+      typeid(Distance) == typeid(char))
+    found = true;
+  if (numeric_limits<wchar_t>::is_signed &&
+      typeid(Distance) == typeid(wchar_t))
+    found = true;
   
-  Distance d = 0;
-  barney(d);
+  VERIFY( found );
 }
 
 int main()
